import gab.opencv.*;
import processing.video.*;
import java.awt.*;

//-----------------------------------------------------------
String profile_img = "profile.jpg";
String profile_name = "Sunao Hashimoto";
String profile_feature = "Game Addict";
String profile_age = "Age: 32";
String profile_occupation = "Occupation: Programmer";
String profile_income = "Income: $50,000";
//-----------------------------------------------------------

Capture video;
OpenCV opencv;
PFont myFont;
CyberText[] cts;

boolean no_target= true;
float rot_angle = 0;
float easing = 0.2;
float cx = 0;
float cy = 0;
float targetX = 0;
float targetY = 0;
PImage img_profile;
float len = 0;

void setup() {
  size(640, 480);
  video = new Capture(this, 320, 240);
  opencv = new OpenCV(this, video.width, video.height);
  opencv.loadCascade(OpenCV.CASCADE_FRONTALFACE);  
  video.start();
  frameRate(60);

  img_profile = loadImage(profile_img);
  cts = new CyberText[10];
  for (int i=0; i<cts.length; i++) {
    cts[i] = new CyberText();
  }
  
  // dummy for loading font
  text("",0,0);
}

void draw() {
  opencv.loadImage(video);
  image(video, 0, 0, width, height );

  noFill();
  stroke(0, 255, 0);
  strokeWeight(3);
  Rectangle[] faces = opencv.detect();

  float fsize = 0;
  for (int i = 0; i < faces.length; i++) {
    float scale = width*1.0/video.width;
    fsize = faces[i].width * scale;
    targetX = (faces[i].x + faces[i].width/2) * scale;
    targetY = (faces[i].y + faces[i].height/2) * scale;
  }
  
  drawCursor(cx, cy, fsize, rot_angle);  

  if (len>0) {
    pushStyle();
    stroke(255);
    strokeWeight(2);
    line( cx + fsize/2, cy+fsize/2, cx + fsize/2 + len, cy+fsize/2 + len );
    popStyle();
  }

  float dx = targetX - cx;
  float dy = targetY - cy;

  if ( rot_angle>-45 ) {  
    rot_angle -= 10.0;
  } else {
    rot_angle = -45;
  }
  if ( dist(targetX, targetY, cx, cy)>10 ) {
    rot_angle = 0;
    len = 0;

    cts[0].reset();
    cts[1].reset();
    cts[2].reset();
    cts[3].reset();
    cts[4].reset();
  } else {
    len += 8;
    if (len>50) {
      len = 50;
    }
  }

  if ( len>=50 ) {
    drawProfile(cx + fsize/2 + len, cy+fsize/2 + len, img_profile);
  }
  if ( abs(dx) > 1 ) {
    cx += dx * easing;
  }
  if ( abs(dy) > 1 ) {
    cy += dy * easing;
  }
}

void captureEvent(Capture c) {
  c.read();
}

void keyPressed() {
  if (key==' ') {
    save("screen.png");
  }
}

void drawProfile(float px, float py, PImage img) {
  translate(px, py);

  pushMatrix();
  noStroke();
  translate(-2, -50);
  fill(0);
  rect( 0, 0, 120, 22 );
  fill(255);
  textSize(12);
  text("Profiler_App", 5, 15 );    
  popMatrix();

  pushMatrix();
  noStroke();
  translate(-2, -28);
  fill(0);
  rect( 0, 0, 85, 22 );
  fill(255);
  textSize(12);
  text("v0.2xx[beta]", 5, 15 );    
  popMatrix();
  
  strokeWeight(4);
  stroke(0);
  rect(0, 0, 100, 100);
  image(img, 0, 0, 100, 100 );

  pushMatrix();
  noStroke();
  translate(-2, 105);
  fill(0);
  rect( 0, 0, 102, 16 );
  fill(255);
  textSize(17);
  text("***-**-****", 5, 15 );    

  fill(0);
  translate(0, 16);
  rect( 0, 0, 80, 16 );
  popMatrix();
  
  noStroke();
  translate(100+5, 0);  
  fill(255);
  rect( 0, 0, 160, 25 );

  fill(0);
  textSize(15);
  cts[0].setText(profile_name);
  cts[0].draw(5, 20, 1);

  translate(0, 28);
  fill(255);
  beginShape();
  vertex(0, 0);
  vertex(140, 0);
  vertex(140, 30);
  vertex(0, 30);
  vertex(-15, 15);
  endShape(CLOSE);

  fill(0);
  textSize(20);
  cts[1].setText(profile_feature);
  cts[1].draw(5, 22);

  translate(0, 30);
  fill(0);
  rect( 0, 0, 70, 25 );
  fill(255);
  textSize(15);
  cts[2].setText(profile_age);
  cts[2].draw(5, 20);

  translate(0, 26);
  fill(0);
  rect( 0, 0, 200, 25 );
  fill(255);
  textSize(15);
  cts[3].setText(profile_occupation);
  cts[3].draw(5, 20);

  translate(0, 26);
  fill(0);
  rect( 0, 0, 150, 25 );
  fill(255);
  textSize(15);
  cts[4].setText(profile_income);
  cts[4].draw(5, 20);
}

void drawCursor(float px, float py, float size, float angle) {   
  if (size<1) return;

  float L = size*0.24;
  float d = (size-3*L)/2.0;

  pushMatrix();
  translate(px, py);
  rotate(radians(angle));
  translate(-size/2, -size/2);
  pushStyle();
  stroke(255);
  strokeWeight(5);  
  line(0, 0, L, 0);
  line(0, 0, 0, L);
  line(size, 0, size-L, 0);
  line(size, 0, size, L);      
  line(0, size, 0, size-L);
  line(0, size, L, size);
  line(size, size, size-L, size);
  line(size, size, size, size-L);

  if ( int(angle)==-45) { 
    line( L+d, 0, L*2+d, 0);
    line( L+d, size, L*2+d, size);
    line( 0, L+d, 0, L*2+d);
    line( size, L+d, size, L*2+d);
  }

  popStyle();
  popMatrix();
}


